@@ -54,6 +54,17 @@ extern void rb_marshal_BOOLEAN__POINTER (GClosure     *closure,
                                          gpointer      invocation_hint,
                                          gpointer      marshal_data);
 
+/* VOID:DOUBLE (./rb-marshal.list:7) */
+#define rb_marshal_VOID__DOUBLE	g_cclosure_marshal_VOID__DOUBLE
+
+/* VOID:POINTER,INT,POINTER (./rb-marshal.list:8) */
+extern void rb_marshal_VOID__POINTER_INT_POINTER (GClosure     *closure,
+                                                  GValue       *return_value,
+                                                  guint         n_param_values,
+                                                  const GValue *param_values,
+                                                  gpointer      invocation_hint,
+                                                  gpointer      marshal_data);
+
 G_END_DECLS
 
 #endif /* __rb_marshal_MARSHAL_H__ */
