@@ -937,7 +937,7 @@ rb_playlist_add_directory (RBPlaylist *playlist, const char *url,
 	info = gnome_vfs_file_info_new ();
 	res = gnome_vfs_directory_read_next (handle, info);
 	while (res == GNOME_VFS_OK) {
-		char *fullpath;
+		char *str, *fullpath;
 
 		if (info->name != NULL && (strcmp (info->name, ".") == 0
 					|| strcmp (info->name, "..") == 0)) {
@@ -945,13 +945,18 @@ rb_playlist_add_directory (RBPlaylist *playlist, const char *url,
 			continue;
 		}
 
-		fullpath = g_build_filename (G_DIR_SEPARATOR_S,
+		str = g_build_filename (G_DIR_SEPARATOR_S,
 					     url, info->name, NULL);
-		if (rb_playlist_parse (playlist, strstr (fullpath, "://")
-				       ? fullpath + 1 : fullpath) != FALSE) {
-			retval = TRUE;
-		}
-		g_free (fullpath);
+		if (strstr (str, "://") != NULL)
+			fullpath = str + 1;
+		else
+			fullpath = str;
+
+		if (rb_playlist_parse (playlist, fullpath) == FALSE)
+			rb_playlist_add_one_url (playlist, fullpath, NULL);
+
+		retval = TRUE;
+		g_free (str);
 		res = gnome_vfs_directory_read_next (handle, info);
 	}
 
@@ -1044,7 +1049,7 @@ rb_playlist_parse (RBPlaylist *playlist, const char *url)
 }
 
 gboolean
-rb_playlist_can_handle (RBPlaylist *playlist, const char *url)
+rb_playlist_can_handle (const char *url)
 {
 	const char *mimetype;
 	int i;
