@@ -1,4 +1,6 @@
 /*
+ *  arch-tag: Header for totally random functions that didn't fit elsewhere
+ *
  *  Copyright (C) 2003 Colin Walters <walters@verbum.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -15,7 +17,6 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: rb-util.h,v 1.1 2003/07/28 21:11:38 cwalters Exp $
  */
 
 #ifndef __RB_UTIL_H
@@ -23,12 +24,15 @@
 
 #include <stdarg.h>
 #include <glib/gtypes.h>
+#include <glib-object.h>
 
 G_BEGIN_DECLS
 
 gboolean rb_true_function (gpointer dummy, ...);
 gboolean rb_false_function (gpointer dummy, ...);
 
+gboolean rb_gvalue_compare (GValue *a, GValue *b);
+
 G_END_DECLS
 
 #endif /* __RB_UTIL_H */
