@@ -208,13 +208,15 @@ tick_timeout (MonkeyMediaPlayer *mp)
 static void
 monkey_media_player_init (MonkeyMediaPlayer *mp)
 {
+	gint ms_period = 1000 / MONKEY_MEDIA_PLAYER_TICK_HZ;
+
 	mp->priv = g_new0 (MonkeyMediaPlayerPrivate, 1);
 
 #ifdef HAVE_AUDIOCD
 	mp->priv->cd = monkey_media_audio_cd_new (NULL);
 #endif	
 
-	mp->priv->tick_timeout_id = g_timeout_add (200, (GSourceFunc) tick_timeout, mp);
+	mp->priv->tick_timeout_id = g_timeout_add (ms_period, (GSourceFunc) tick_timeout, mp);
 }
 
 static void
@@ -468,9 +470,11 @@ static void
 monkey_media_player_construct (MonkeyMediaPlayer *mp,
 			       gboolean iradio_mode,
 			       gboolean audiocd_mode,
+			       const char *uri,
 			       GError **error)
 {
 	GstDParamManager *dpman;
+	char *decoder_name = NULL;
 
 	/* The main playback pipeline for iradio, at the end this looks like:
 	 * { { src ! queue } ! { mad ! volume ! sink } }
@@ -513,6 +517,7 @@ monkey_media_player_construct (MonkeyMediaPlayer *mp,
 			     _("Failed to create %s input element; check your installation"),
 			     audiocd_mode ? "cdparanoia" : "gnomevfssrc");
 		gst_object_unref (GST_OBJECT (mp->priv->pipeline));
+		mp->priv->pipeline = NULL;
 		return;
 	}
 	gst_bin_add (GST_BIN (mp->priv->srcthread), mp->priv->src);
@@ -537,6 +542,7 @@ monkey_media_player_construct (MonkeyMediaPlayer *mp,
 				     MONKEY_MEDIA_PLAYER_ERROR_NO_QUEUE_PLUGIN,
 				     _("Failed to create queue element; check your installation"));
 			gst_object_unref (GST_OBJECT (mp->priv->pipeline));
+			mp->priv->pipeline = NULL;
 			return;
 		}
 #if GST_VERSION_MAJOR == 0 && GST_VERSION_MINOR == 6
@@ -550,17 +556,34 @@ monkey_media_player_construct (MonkeyMediaPlayer *mp,
 	}
 
 	/* The decoding element */
-	if (iradio_mode) {
-		mp->priv->decoder = gst_element_factory_make ("mad", "autoplugger");
-	} else
-		mp->priv->decoder = gst_element_factory_make ("spider", "autoplugger");
+	if (iradio_mode)
+#ifdef HAVE_MP3
+		decoder_name = "mad";
+#else
+		decoder_name = "vorbisfile";
+#endif
+#if GST_VERSION_MAJOR == 0 && GST_VERSION_MINOR == 6
+	else if (g_str_has_suffix (uri, ".mp3"))
+		decoder_name = "mad";
+	else if (g_str_has_suffix (uri, ".ogg"))
+		decoder_name = "vorbisfile";
+	else if (g_str_has_suffix (uri, ".flac"))
+		decoder_name = "flacdec";
+#endif
+	else
+		decoder_name = "spider";
 
+	mp->priv->decoder = gst_element_factory_make (decoder_name, "autoplugger");
 	if (mp->priv->decoder == NULL) {
+		char *err = g_strdup_printf (_("Failed to create %s element; check your installation"),
+					     decoder_name);
 		g_set_error (error,
 			     MONKEY_MEDIA_PLAYER_ERROR,
 			     MONKEY_MEDIA_PLAYER_ERROR_NO_DEMUX_PLUGIN,
-			     _("Failed to create spider element; check your installation"));
+			     err);
+		g_free (err);
 		gst_object_unref (GST_OBJECT (mp->priv->pipeline));
+		mp->priv->pipeline = NULL;
 		return;
 	}
 	gst_bin_add (GST_BIN (mp->priv->waiting_bin), mp->priv->decoder);
@@ -573,6 +596,7 @@ monkey_media_player_construct (MonkeyMediaPlayer *mp,
 			     MONKEY_MEDIA_PLAYER_ERROR_NO_VOLUME_PLUGIN,
 			     _("Failed to create volume element; check your installation"));
 		gst_object_unref (GST_OBJECT (mp->priv->pipeline));
+		mp->priv->pipeline = NULL;
 		return;
 	}
 	gst_bin_add (GST_BIN (mp->priv->waiting_bin), mp->priv->volume);
@@ -591,6 +615,7 @@ monkey_media_player_construct (MonkeyMediaPlayer *mp,
 			     MONKEY_MEDIA_PLAYER_ERROR_NO_AUDIO,
 			     _("Could not create audio output element; check your settings"));
 		gst_object_unref (GST_OBJECT (mp->priv->pipeline));
+		mp->priv->pipeline = NULL;
 		return;
 	}
 	gst_bin_add (GST_BIN (mp->priv->waiting_bin), mp->priv->sink);
@@ -671,8 +696,8 @@ monkey_media_player_open (MonkeyMediaPlayer *mp,
 			  const char *uri,
 			  GError **error)
 {
-	gboolean audiocd_mode = uri && !strncmp ("audiocd://", uri, 10);
-	gboolean iradio_mode = uri && !strncmp ("http://", uri, 7);
+	gboolean audiocd_mode = uri && g_str_has_prefix (uri, "audiocd://");
+	gboolean iradio_mode = uri && g_str_has_prefix (uri, "http://");
 
 	g_return_if_fail (MONKEY_MEDIA_IS_PLAYER (mp));
 
@@ -691,7 +716,7 @@ monkey_media_player_open (MonkeyMediaPlayer *mp,
 		return;
 	}
 
-	monkey_media_player_construct (mp, iradio_mode, audiocd_mode, error);
+	monkey_media_player_construct (mp, iradio_mode, audiocd_mode, uri, error);
 	if (error && *error)
 		return;
 
