@@ -67,7 +67,14 @@ void			rhythmdb_query_model_add_entry	(RhythmDBQueryModel *model, RhythmDBEntry
 
 void			rhythmdb_query_model_remove_entry	(RhythmDBQueryModel *model, RhythmDBEntry *entry);
 
-void			rhythmdb_query_model_complete	(RhythmDBQueryModel *model);
+void			rhythmdb_query_model_set_connected	(RhythmDBQueryModel *model, gboolean connected);
+
+void			rhythmdb_query_model_signal_complete	(RhythmDBQueryModel *model);
+void			rhythmdb_query_model_finish_complete	(RhythmDBQueryModel *model);
+
+GnomeVFSFileSize	rhythmdb_query_model_get_size	(RhythmDBQueryModel *model);
+
+long			rhythmdb_query_model_get_duration(RhythmDBQueryModel *model);
 
 G_END_DECLS
 
