@@ -678,6 +678,8 @@ rhythmdb_add_song (RhythmDB *db, const char *uri, GError **error)
 	rhythmdb_write_lock (db);
 
 	entry = rhythmdb_entry_new (db, RHYTHMDB_ENTRY_TYPE_SONG, realuri);
+	if (entry == NULL)
+		goto out_dupentry;
 	synchronize_entry_with_data (db, entry, metadata);
 
 	/* initialize the last played date to 0=never */
@@ -686,6 +688,7 @@ rhythmdb_add_song (RhythmDB *db, const char *uri, GError **error)
 	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_LAST_PLAYED, &last_time);
 	g_value_unset (&last_time);
 
+ out_dupentry:
 	g_free (metadata);
 	g_free (realuri);
 
@@ -797,7 +800,6 @@ update_song (RhythmDB *db, RhythmDBEntry *entry, GError **error)
 	time_t stored_mtime;
 	GnomeVFSFileInfo *vfsinfo = NULL;
 	GnomeVFSResult result;
-	RhythmDBEntryUpdateData *metadata;
 
 	rhythmdb_read_lock (db);
 	location = rhythmdb_entry_get_string (db, entry, RHYTHMDB_PROP_LOCATION);
@@ -823,16 +825,10 @@ update_song (RhythmDB *db, RhythmDBEntry *entry, GError **error)
 	if (stored_mtime == vfsinfo->mtime)
 		goto out;
 
-	metadata = read_metadata (location, error);
-	if (!metadata) {
-		rb_debug ("failed to read data from \"%s\"", location);
-		goto out;
-	}
-
 	rhythmdb_write_lock (db);
 	
-	synchronize_entry_with_data (db, entry, metadata);
-	g_free (metadata);
+	rhythmdb_entry_delete (db, entry);
+	rhythmdb_add_uri_async (db, location); 
 	rhythmdb_entry_unref_unlocked (db, entry);
 
 	rhythmdb_write_unlock (db);
@@ -1917,8 +1913,10 @@ rhythmdb_compute_status_normal (gint n_songs, glong duration, GnomeVFSFileSize s
 			const char *hourfmt = ngettext ("%ld hour", "%ld hours", hours);
 			char *fmt;
 			if (minutes > 0) {
+				/* Translators: the format is "X hours and X minutes" */
 				fmt = g_strdup_printf (_("%s and %s"), hourfmt, minutefmt);
 			} else {
+				/* Translators: the format is "X hours" */
 				fmt = g_strdup_printf ("%s", hourfmt);
 			}
 			time = g_strdup_printf (fmt, hours, minutes);
