@@ -28,6 +28,7 @@
 #include <stdarg.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <libgnomevfs/gnome-vfs-file-info.h>
+#include <libxml/tree.h>
 
 #include "config.h"
 
@@ -53,6 +54,7 @@ typedef enum
 	RHYTHMDB_QUERY_SUBQUERY,
 	RHYTHMDB_QUERY_PROP_EQUALS,
 	RHYTHMDB_QUERY_PROP_LIKE,
+	RHYTHMDB_QUERY_PROP_NOT_LIKE,
 	RHYTHMDB_QUERY_PROP_GREATER,
 	RHYTHMDB_QUERY_PROP_LESS,
 } RhythmDBQueryType;
@@ -189,6 +191,9 @@ RhythmDBEntry *	rhythmdb_add_song	(RhythmDB *db, const char *uri, GError **error
 void		rhythmdb_entry_set	(RhythmDB *db, RhythmDBEntry *entry,
 					 guint propid, GValue *value);
 
+void		rhythmdb_entry_queue_set(RhythmDB *db, RhythmDBEntry *entry,
+					 guint propid, GValue *value);
+
 void		rhythmdb_entry_get	(RhythmDB *db, RhythmDBEntry *entry,
 					 guint propid, GValue *value);
 
@@ -264,6 +269,14 @@ void		rhythmdb_query_append			(RhythmDB *db, GPtrArray *query, ...);
 void		rhythmdb_query_free			(GPtrArray *query);
 GPtrArray *	rhythmdb_query_copy			(GPtrArray *array);
 
+void		rhythmdb_query_serialize		(RhythmDB *db, GPtrArray *query,
+							 xmlNodePtr node);
+
+GPtrArray *	rhythmdb_query_deserialize		(RhythmDB *db, xmlNodePtr node);
+
+inline const char *	rhythmdb_nice_elt_name_from_propid	(RhythmDB *db, gint propid);
+inline int		rhythmdb_propid_from_nice_elt_name	(RhythmDB *db, const char *name);
+
 void		rhythmdb_emit_entry_added		(RhythmDB *db, RhythmDBEntry *entry);
 void		rhythmdb_emit_entry_restored		(RhythmDB *db, RhythmDBEntry *entry);
 void		rhythmdb_emit_entry_deleted		(RhythmDB *db, RhythmDBEntry *entry);
