@@ -142,11 +142,18 @@ static void
 rb_history_finalize (GObject *object)
 {
 	RBHistory *hist;
+	GList *cur;
 	g_return_if_fail (object != NULL);
 	g_return_if_fail (RB_IS_HISTORY (object));
 
 	hist = RB_HISTORY (object);
 
+	/* unref all of the stored entries */
+	for (cur = hist->priv->head; cur; cur = g_list_next (cur)) {
+		rhythmdb_entry_unref (hist->priv->db, cur->data);
+	}
+
+	g_hash_table_destroy (hist->priv->entry_to_link);
 	g_list_free (hist->priv->head);
 	g_free (hist->priv);
 
@@ -262,6 +269,12 @@ rb_history_prepend (RBHistory *hist, RhythmDBEntry *entry)
 	rb_history_limit_size (hist, FALSE, MAX_HISTORY_SIZE);
 }
 
+void
+rb_history_clear (RBHistory *hist)
+{
+	rb_history_limit_size (hist, TRUE, 1);
+}
+
 /**
  * Cuts nodes off of the history from the desired end until it is smaller than max_size. 
  * Never cuts off the current node.
