@@ -45,11 +45,13 @@
 #include "rb-glade-helpers.h"
 #include "rb-bonobo-helpers.h"
 #include "rb-thread-helpers.h"
+#include "rb-file-helpers.h"
 #include "rb-cut-and-paste-code.h"
 #include "rb-dialog.h"
 #include "rb-preferences.h"
 #include "rb-debug.h"
 #include "rb-player.h"
+#include "rb-playlist.h"
 #include "rb-volume.h"
 #include "rb-remote.h"
 #include "eel-gconf-extensions.h"
@@ -133,6 +135,8 @@ static void rb_shell_player_sync_volume (RBShellPlayer *player);
 static void tick_cb (MonkeyMediaPlayer *player, long elapsed, gpointer data);
 static void eos_cb (MonkeyMediaPlayer *player, gpointer data);
 static void error_cb (MonkeyMediaPlayer *player, GError *err, gpointer data);
+static void rb_shell_player_error (RBShellPlayer *player, GError *err,
+				   gboolean lock);
 
 static void cancel_buffering_dialog (RBShellPlayer *player);
 
@@ -196,6 +200,8 @@ struct RBShellPlayerPrivate
 	GtkWidget *buffering_progress;
 	guint buffering_progress_idle_id;
 
+	GError *playlist_parse_error;
+
 	GtkTooltips *tooltips;
 	GtkWidget *prev_button;
 	PlayButtonState playbutton_state;
@@ -723,6 +729,16 @@ rb_shell_player_get_property (GObject *object,
 	}
 }
 
+GQuark
+rb_shell_player_error_quark (void)
+{
+	static GQuark quark = 0;
+	if (!quark)
+		quark = g_quark_from_static_string ("rb_shell_player_error");
+
+	return quark;
+}
+
 void
 rb_shell_player_set_source (RBShellPlayer *player,
 			    RBSource *source)
@@ -780,6 +796,31 @@ rb_shell_player_have_first (RBShellPlayer *player, RBSource *source)
 	return FALSE;
 }
 
+static void
+rb_shell_player_open_playlist_location (RBPlaylist *playlist, const char *uri,
+					const char *title, const char *genre,
+					RBShellPlayer *player)
+{
+	GError *error = NULL;
+
+	if (monkey_media_player_playing (player->priv->mmplayer))
+		return;
+
+	monkey_media_player_open (player->priv->mmplayer, uri, &error);
+	if (error != NULL) {
+		if (player->priv->playlist_parse_error != NULL) {
+			g_error_free (player->priv->playlist_parse_error);
+			player->priv->playlist_parse_error = NULL;
+		}
+		player->priv->playlist_parse_error = g_error_copy (error);
+		return;
+	}
+
+	monkey_media_player_play (player->priv->mmplayer);
+
+	g_object_notify (G_OBJECT (player), "playing");
+}
+
 static void
 rb_shell_player_open_location (RBShellPlayer *player,
 			       const char *location,
@@ -812,6 +853,30 @@ rb_shell_player_open_location (RBShellPlayer *player,
 	was_playing = monkey_media_player_playing (player->priv->mmplayer);
 
 	monkey_media_player_close (player->priv->mmplayer);
+
+	if (rb_uri_is_iradio (location) != FALSE
+	    && rb_playlist_can_handle (location) != FALSE) {
+		RBPlaylist *playlist;
+
+		playlist = rb_playlist_new ();
+		g_signal_connect_object (G_OBJECT (playlist), "entry",
+					 G_CALLBACK (rb_shell_player_open_playlist_location),
+					 player, 0);
+		if (rb_playlist_parse (playlist, location) == FALSE) {
+			g_object_unref (playlist);
+			goto normal_open;
+		}
+		g_object_unref (playlist);
+		if (!monkey_media_player_playing (player->priv->mmplayer)) {
+			if (error) {
+				*error = g_error_copy (player->priv->playlist_parse_error);
+				g_error_free (player->priv->playlist_parse_error);
+				player->priv->playlist_parse_error = NULL;
+			}
+		}
+		return;
+	}
+ normal_open:
 	monkey_media_player_open (player->priv->mmplayer, location, error);
 	if (error && *error)
 		return;
@@ -1060,12 +1125,18 @@ rb_shell_player_playpause (RBShellPlayer *player)
 		entry = rb_shell_player_get_playing_entry (player);
 		if (entry == NULL) {
 			RBEntryView *songs = rb_source_get_entry_view (player->priv->source);
-			
-			if (eel_gconf_get_boolean (CONF_STATE_SHUFFLE)) {
-				rb_debug ("choosing random entry");
-				entry =  rb_entry_view_get_random_entry (songs);
+
+			GList* selection = rb_entry_view_get_selected_entries (songs);
+			if (selection != NULL) {
+				rb_debug ("choosing first selected entry");
+				entry = (RhythmDBEntry*) selection->data;
 			} else {
-				entry = rb_entry_view_get_first_entry (songs);
+				if (eel_gconf_get_boolean (CONF_STATE_SHUFFLE)) {
+					rb_debug ("choosing random entry");
+					entry = rb_entry_view_get_random_entry (songs);
+				} else {
+					entry = rb_entry_view_get_first_entry (songs);
+				}
 			}
 			if (entry != NULL)
 				rb_shell_player_set_playing_entry (player, entry);
@@ -1363,11 +1434,11 @@ rb_shell_player_sync_buttons (RBShellPlayer *player)
 	PlayButtonState pstate = PLAY_BUTTON_PLAY;
         gboolean not_small;
 
-	rb_debug ("syncing with source %p", source);
-
 	source = rb_shell_player_get_playing_entry (player) == NULL ?
 		 player->priv->selected_source : player->priv->source;
 
+	rb_debug ("syncing with source %p", source);
+
 	/* If we have a source and it's not empty, next and prev depend
 	 * on the availability of the next/prev entry. However if we are 
 	 * shuffling only next make sense and if we are repeating next
@@ -1611,21 +1682,15 @@ eos_cb (MonkeyMediaPlayer *mmplayer, gpointer data)
 }
 
 static void
-error_cb (MonkeyMediaPlayer *mmplayer, GError *err, gpointer data)
+rb_shell_player_error (RBShellPlayer *player, GError *err, gboolean lock)
 {
-	RBShellPlayer *player = RB_SHELL_PLAYER (data);
-	if (player->priv->handling_error)
-	{
+	if (player->priv->handling_error) {
 		rb_debug ("ignoring error: %s", err->message);
 		return;
 	}
 
-	gdk_threads_enter ();
-
-	if (!monkey_media_player_playing (mmplayer)) {
-		rb_debug ("mmplayer is not playing, ignoring error");
-		goto out_unlock;
-	}
+	if (lock != FALSE)
+		GDK_THREADS_ENTER ();
 
 	cancel_buffering_dialog (player);
 	rb_debug ("error: %s", err->message);
@@ -1635,8 +1700,14 @@ error_cb (MonkeyMediaPlayer *mmplayer, GError *err, gpointer data)
 	player->priv->handling_error = FALSE;
 	rb_debug ("exiting error hander");
 
- out_unlock:
-	gdk_threads_leave ();
+	if (lock != FALSE)
+		GDK_THREADS_LEAVE ();
+}
+
+static void
+error_cb (MonkeyMediaPlayer *mmplayer, GError *err, gpointer data)
+{
+	rb_shell_player_error (RB_SHELL_PLAYER (data), err, TRUE);
 }
 
 static void
@@ -1861,7 +1932,8 @@ buffering_end_cb (MonkeyMediaPlayer *mmplayer,
 	gdk_threads_enter ();
 
 	cancel_buffering_dialog (player);
-	rb_source_buffering_done (player->priv->source);
+	if (player->priv->source)
+		rb_source_buffering_done (player->priv->source);
 
 	gdk_threads_leave ();
 }
