@@ -574,8 +574,8 @@ rb_shell_corba_handle_file (PortableServer_Servant _servant,
 		return;
 	}
 
-	parser = rb_playlist_new ();	
-	if (rb_playlist_can_handle (parser, uri)) {
+	parser = rb_playlist_new ();
+	if (rb_playlist_can_handle (uri)) {
 		uri = rb_playlist_manager_parse_file (shell->priv->playlist_manager, uri);
 	} else 
 		rhythmdb_add_uri_async (shell->priv->db, uri);
@@ -762,7 +762,7 @@ rb_shell_corba_get_player_properties (PortableServer_Servant _servant,
 	 * return a corba exception
 	 */
 	
-	return CORBA_Object_duplicate (BONOBO_OBJREF (shell->priv->pb), NULL);
+	return bonobo_object_dup_ref (BONOBO_OBJREF (shell->priv->pb), NULL);
 }
 
 static void
@@ -1468,9 +1468,11 @@ rb_shell_set_duration (RBShell *shell, const char *duration)
 	if (shell->priv->cached_title == NULL) 
 		tooltip = g_strdup (_("Not playing"));
 	else if (!playing) {
+		/* Translators: the first %s is substituted by the song name, the second one is the elapsed and total time */
 		tooltip = g_strdup_printf (_("%s\nPaused, %s"),
 					 shell->priv->cached_title, duration);
 	} else {
+		/* Translators: the first %s is substituted by the song name, the second one is the elapsed and total time */
 		tooltip = g_strdup_printf (_("%s\n%s"),
 					   shell->priv->cached_title, duration);
 	}
@@ -1505,6 +1507,7 @@ rb_shell_set_window_title (RBShell *shell, const char *window_title)
 
 		rb_debug ("setting title to \"%s\"", window_title);
 		if (!playing) {
+			/* Translators: %s is the song name */
 			title = g_strdup_printf (_("%s (Paused)"), window_title);
 			gtk_window_set_title (GTK_WINDOW (shell->priv->window),
 					      title);
@@ -1596,19 +1599,22 @@ rb_shell_cmd_about (BonoboUIComponent *component,
 #ifdef HAVE_GSTREAMER
 		backend = "GStreamer";
 #else
-		backend = "Xine";
+		backend = "xine-lib";
 #endif		
 #ifdef HAVE_MP3
-		g_string_append (formats, "MP3 ");
+		g_string_append (formats, " MP3");
 #endif
 #ifdef HAVE_VORBIS
-		g_string_append (formats, "Vorbis ");
+		g_string_append (formats, " Vorbis");
 #endif
 #ifdef HAVE_FLAC
-		g_string_append (formats, "FLAC ");
+		g_string_append (formats, " FLAC");
 #endif
-		
-		comment = g_strdup_printf (_("Music management and playback software for GNOME.\nAudio backend: %s\nAudio formats: %s\n"), backend, formats->str);
+#ifdef HAVE_MP4
+		g_string_append (formats, " MPEG-4");
+#endif
+
+		comment = g_strdup_printf (_("Music management and playback software for GNOME.\nAudio backend: %s\nAudio formats:%s\n"), backend, formats->str);
 
 		g_string_free (formats, TRUE);
 	}
@@ -1933,10 +1939,15 @@ sourcelist_drag_received_cb (RBSourceList *sourcelist,
 			     GtkSelectionData *data,
 			     RBShell *shell)
 {
+	if (source == NULL) 
+		source = rb_playlist_manager_new_playlist (shell->priv->playlist_manager, 
+							   FALSE);
+	
 	if (source != NULL) {
 		rb_source_receive_drag (source, data);
 		return;
-	}
+	} 
+
 
 /* 	if (data->type == gdk_atom_intern (RB_LIBRARY_DND_NODE_ID_TYPE, TRUE)) { */
 /* 		long id; */
@@ -2003,7 +2014,7 @@ rb_shell_jump_to_entry_with_source (RBShell *shell, RBSource *source,
 	}
 
 	songs = rb_source_get_entry_view (source);
-	if (!rb_entry_view_get_entry_visible (songs, entry)) {
+	if (!rb_entry_view_get_entry_contained (songs, entry)) {
 		rb_source_reset_filters (source);
 		rb_source_header_clear_search (shell->priv->source_header);
 	}
@@ -2013,7 +2024,7 @@ rb_shell_jump_to_entry_with_source (RBShell *shell, RBSource *source,
 	while (gtk_events_pending ())
 		gtk_main_iteration ();
 
-	if (!rb_entry_view_get_entry_visible (songs, entry)) {
+	if (!rb_entry_view_get_entry_contained (songs, entry)) {
 		rb_source_search (shell->priv->selected_source, NULL);
 	}
 
