@@ -2,7 +2,7 @@
  *  arch-tag: Implementation of the abstract base class of all sources
  *
  *  Copyright (C) 2002 Jorn Baayen <jorn@nl.linux.org>
- *  Copyright (C) 2003 Colin Walters <cwalters@gnome.org>
+ *  Copyright (C) 2003 Colin Walters <walters@gnome.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -282,19 +282,20 @@ rb_source_notify_filter_changed (RBSource *status)
 void
 rb_source_update_play_statistics (RBSource *source, RhythmDB *db, RhythmDBEntry *entry)
 {
-	char *time_string;
 	time_t now;
+	guint current_count;
 	GValue value = { 0, };
 
 	g_value_init (&value, G_TYPE_INT);
 
-	rhythmdb_write_lock (db);
-	
-	g_value_set_int (&value, rhythmdb_entry_get_int (db, entry,
-							 RHYTHMDB_PROP_PLAY_COUNT) + 1);
+	rhythmdb_read_lock (db);
+	current_count = rhythmdb_entry_get_int (db, entry, RHYTHMDB_PROP_PLAY_COUNT);
+	rhythmdb_read_unlock (db);
+
+	g_value_set_int (&value, current_count + 1);
 
 	/* Increment current play count */
-	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_PLAY_COUNT, &value);
+	rhythmdb_entry_queue_set (db, entry, RHYTHMDB_PROP_PLAY_COUNT, &value);
 	g_value_unset (&value);
 	
 	/* Reset the last played time */
@@ -302,18 +303,8 @@ rb_source_update_play_statistics (RBSource *source, RhythmDB *db, RhythmDBEntry
 
 	g_value_init (&value, G_TYPE_LONG);
 	g_value_set_long (&value, now);
-	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_LAST_PLAYED, &value);
+	rhythmdb_entry_queue_set (db, entry, RHYTHMDB_PROP_LAST_PLAYED, &value);
 	g_value_unset (&value);
-
-	time_string = eel_strdup_strftime (_("%Y-%m-%d %H:%M"), localtime (&now));
-
-	g_value_init (&value, G_TYPE_STRING);
-	g_value_set_string (&value, time_string);
-	g_free (time_string);
-	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_LAST_PLAYED_STR, &value);
-	g_value_unset (&value);
-
-	rhythmdb_write_unlock (db);
 }
 
 RBEntryView *
@@ -437,7 +428,7 @@ rb_source_cut (RBSource *source)
 static GList *
 default_copy (RBSource *source)
 {
-	return g_list_copy (rb_entry_view_get_selected_entries (rb_source_get_entry_view (source)));
+	return rb_entry_view_get_selected_entries (rb_source_get_entry_view (source));
 }
 	
 GList *
