@@ -583,6 +583,7 @@ rb_entry_view_set_property (GObject *object,
 		gtk_tree_view_set_model (GTK_TREE_VIEW (view->priv->treeview),
 					 GTK_TREE_MODEL (new_model));
 		view->priv->model = new_model;
+		view->priv->have_selection = FALSE;
 		queue_changed_sig (view);
 
 		break;
@@ -1385,7 +1386,6 @@ rb_entry_view_constructor (GType type, guint n_construct_properties,
 
 	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (view->priv->treeview), TRUE);
 	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (view->priv->treeview), TRUE);
-	gtk_tree_view_set_reorderable (GTK_TREE_VIEW (view->priv->treeview), TRUE);
 	gtk_tree_selection_set_mode (view->priv->selection, GTK_SELECTION_MULTIPLE);
 	
 	if (view->priv->is_drag_source)
@@ -1396,7 +1396,7 @@ rb_entry_view_constructor (GType type, guint n_construct_properties,
 						     GDK_ACTION_COPY | GDK_ACTION_MOVE);
 	if (view->priv->is_drag_dest)
 		rb_tree_dnd_add_drag_dest_support (GTK_TREE_VIEW (view->priv->treeview),
-						   RB_TREE_DEST_CAN_DROP_BETWEEN,
+						   RB_TREE_DEST_CAN_DROP_BETWEEN | RB_TREE_DEST_EMPTY_VIEW_DROP,
 						   rb_entry_view_drag_types,
 						   G_N_ELEMENTS (rb_entry_view_drag_types),
 						   GDK_ACTION_COPY | GDK_ACTION_MOVE);
@@ -1528,8 +1528,10 @@ rb_entry_view_get_next_from_entry (RBEntryView *view, RhythmDBEntry *entry)
 	g_return_val_if_fail (entry != NULL, NULL);
 
 	if (!rhythmdb_model_entry_to_iter (view->priv->model,
-					   entry, &iter))
-		return NULL;
+					   entry, &iter)) {
+		// If the entry isn't in the entryview, the "next" entry is the first.
+		return rb_entry_view_get_first_entry (view);
+	}
 	
 	if (gtk_tree_model_iter_next (GTK_TREE_MODEL (view->priv->model),
 				      &iter))
@@ -1851,6 +1853,16 @@ rb_entry_view_get_entry_visible (RBEntryView *view,
 	return rb_entry_view_entry_is_visible (view, entry, &unused);
 }
 
+gboolean
+rb_entry_view_get_entry_contained (RBEntryView *view,
+				   RhythmDBEntry *entry)
+{
+	GtkTreeIter unused;
+
+	return rhythmdb_model_entry_to_iter (view->priv->model,
+					     entry, &unused);
+}
+
 static gboolean
 rb_entry_view_entry_is_visible (RBEntryView *view,
 				RhythmDBEntry *entry,
